import numpy as np
import mahotas as mh
def test_find():
    for _ in range(16):
        f = np.random.random((128,128))
        c0,c1 = 43,23
        for h,w in [(12,56),
                    (11,7),
                    (12,7)]:
            matches = mh.find(f, f[c0:c0+h, c1:c1+w])
            coords = np.array(np.where(matches))
            assert np.all(coords.T == np.array((c0,c1)), 1).any()

def test_negative():
    f = 255*np.random.random((228,228))
    f = f.astype(np.uint8)
    h,w = 12,6
    t = f[:h,:w]
    matches = mh.find(f, t)
    coords = np.array(np.where(matches))
    for y,x in zip(*coords):
        if y < 0 or x < 0:
            continue
        assert np.all(f[y:y+h, x:x+w] == t)



# Copyright (C) 2008-2015, Luis Pedro Coelho <luis@luispedro.org>
# vim: set ts=4 sts=4 sw=4 expandtab smartindent:
# 
# License: MIT (see COPYING file)

from __future__ import division

from . import _bbox
import numpy as np

def bbox(img, border=None, as_slice=False):
    """
    min1,max1,min2,max2 = bbox(img, border={0}, as_slice={False})

    Calculate the bounding box of image img.

    Parameters
    ----------
    img : ndarray
        Any integer image type

    Returns
    -------
    min1,max1,min2,max2 : int,int,int,int
        These are such that ``img[min1:max1, min2:max2]`` contains all non-zero
        pixels. Returned when ``as_slice`` is false (the default)
    s : slice
        A slice representation of the bounding box. Returned when ``as_slice``
        is true
    """
    if not img.shape:
        return np.array([], dtype=np.intp)
    r = _bbox.bbox(img)
    if border:
        r = r.reshape((-1, 2))
        np.maximum(r.T[0] - border, 0, out=r.T[0])
        r.T[1] += border
        r = r.ravel()
    if as_slice:
        r = tuple([slice(s,e) for s,e in r.reshape((-1,2))])
    return r

def croptobbox(img, border=None):
    """
    nimg = croptobbox(img, border=0)

    Returns a version of img cropped to the image's bounding box

    Parameters
    ----------
    img : ndarray
        Integer image array
    border : int, optional
        whether to add a border (default no border)

    Returns
    -------
    nimg : ndarray
        A subimage of img.

    Bugs
    ----
    Note that the border is on the bounding box, not on the final image! This
    means that if the image has a positive pixel on its margin, it will still
    be on the margin.

    This ensures that the result is always a sub-image of the input.
    """
    sl = bbox(img, border=border, as_slice=True)
    return img[sl]




import numpy as np
import mahotas as mh
import mahotas.labeled
from nose.tools import raises

def test_border():
    labeled = np.zeros((32,32), np.uint8)
    labeled[8:11] = 1
    labeled[11:14] = 2
    labeled[14:17] = 3
    labeled[10,8:] = 0
    b12 = mahotas.labeled.border(labeled, 1, 2)
    YX = np.where(b12)
    YX = np.array(YX).T
    b13 = mahotas.labeled.border(labeled, 1, 3)

    assert not np.any(b13)
    assert np.any(b12)
    assert (11,0) in YX
    assert (11,1) in YX
    assert (12,1) in YX
    assert (12,9) not in YX

    b13 = mahotas.labeled.border(labeled, 1, 3, always_return=0)
    assert b13 is None

def _included(a,b):
    assert np.sum(a&b) == a.sum()

def test_borders():
    labeled = np.zeros((32,32), np.uint8)
    labeled[8:11] = 1
    labeled[11:14] = 2
    labeled[14:17] = 3
    labeled[10,8:] = 0
    borders = mahotas.labeled.borders(labeled)
    _included(mahotas.labeled.border(labeled,1,2), borders)
    _included(mahotas.labeled.border(labeled,1,23), borders)
    _included(mahotas.labeled.border(labeled,1,3), borders)
    _included(mahotas.labeled.border(labeled,2,3), borders)

    union = np.zeros_like(borders)
    for i in range(4):
        for j in range(4):
            if i != j:
                union |= mahotas.labeled.border(labeled, i, j)

    assert np.all(union == borders)


def slow_labeled_sum(array, labeled):
    return np.array([
            np.sum(array * (labeled == i))
            for i in range(labeled.max()+1)
        ])

def slow_labeled_max(array, labeled):
    return np.array([
            np.max(array * (labeled == i))
            for i in range(labeled.max()+1)
        ])

def slow_labeled_min(array, labeled):
    return np.array([
            np.min(array * (labeled == i))
            for i in range(labeled.max()+1)
        ])

def test_sum_labeled():
    np.random.seed(334)
    for i in range(16):
        f = np.random.random_sample((64,128))
        labeled = np.zeros(f.shape, dtype=np.intc)
        labeled += 8 * np.random.random_sample(labeled.shape).astype(np.intc)
        fast = mahotas.labeled.labeled_sum(f, labeled)
        slow = slow_labeled_sum(f, labeled)
        assert np.allclose(fast, slow)

def test_sum_minlength_arg():
    for _ in range(8):
        f = mh.dilate(np.random.random_sample((64,64)) > .8)
        labeled, n = mh.labeled.label(f)
        sizes = mh.labeled.labeled_sum(f, labeled)
        sizes2 = mh.labeled.labeled_sum(f, labeled, minlength=(n+23))
        assert len(sizes) == (n+1)
        assert len(sizes2) == (n+23)
        assert np.all(sizes == sizes2[:n+1])


def test_max_labeled():
    np.random.seed(334)
    for i in range(16):
        f = np.random.random_sample((64,128))
        labeled = np.zeros(f.shape, dtype=np.intc)
        labeled += 8 * np.random.random_sample(labeled.shape).astype(np.intc)
        fast = mahotas.labeled.labeled_max(f, labeled)
        slow = slow_labeled_max(f, labeled)
        assert np.allclose(fast, slow)

def test_min_labeled():
    np.random.seed(334)
    for i in range(16):
        f = np.random.random_sample((64,128))
        labeled = np.zeros(f.shape, dtype=np.intc)
        labeled += 8 * np.random.random_sample(labeled.shape).astype(np.intc)
        fast = mahotas.labeled.labeled_min(f, labeled)
        slow = slow_labeled_min(f, labeled)
        assert np.allclose(fast, slow)

def slow_labeled_size(labeled):
    return np.array([
            np.sum(labeled == i)
            for i in range(labeled.max()+1)
        ])


def test_size_labeled():
    np.random.seed(334)
    for i in range(16):
        labeled = np.zeros((64,125), dtype=np.intc)
        labeled += 8 * np.random.random_sample(labeled.shape).astype(np.intc)
        fast = mahotas.labeled.labeled_size(labeled)
        slow = slow_labeled_size(labeled)
        assert np.all(fast == slow)

def test_remove_bordering():
    np.random.seed(343)
    for i in range(4):
        labeled,_ = mahotas.label(np.random.random_sample((128,64)) > .7)
        removed = mahotas.labeled.remove_bordering(labeled)
        assert np.all(removed[0] == 0)
        assert np.all(removed[-1] == 0)
        assert np.all(removed[:,0] == 0)
        assert np.all(removed[:,-1] == 0)

        removed2 = np.zeros_like(removed)
        mahotas.labeled.remove_bordering(labeled, out=removed2)
        assert np.all(removed2 == removed)

@raises(ValueError)
def test_check_array_labeled_not_same_shape():
    arr = np.zeros((4,7))
    lab = np.zeros((4,3), dtype=np.intc)
    mahotas.labeled._as_labeled(arr, lab, 'testing')

def _nelems(arr):
    return len(set(map(int, arr.ravel())))

def test_relabel():
    np.random.seed(24)
    for i in range(8):
        f = np.random.random_sample((128,128)) > .8
        labeled, n = mahotas.labeled.label(f)
        labeled *= ( (labeled % 7) != 4)
        relabeled,new_n = mahotas.labeled.relabel(labeled)
        assert relabeled.max() == new_n
        assert (relabeled.max()+1) == _nelems(labeled)
        assert np.all( (relabeled > 0) == (labeled > 0) )
        assert not np.all(labeled == relabeled)

        for a in (8, 23, 35, 13, 213):
            assert _nelems(labeled[relabeled == a]) == 1
            assert _nelems(relabeled[labeled == a]) == 1
        mahotas.labeled.relabel(labeled, inplace=True)
        assert np.all( labeled == relabeled )


def test_remove_regions():
    np.random.seed(34)
    f = np.random.random_sample((128,128)) > .92
    labeled, n = mahotas.labeled.label(f)
    regions = [23,55,8]
    removed = mahotas.labeled.remove_regions(labeled, regions)

    for r in regions:
        assert not  np.any(removed == r)
        assert      np.any(labeled == r)
    mahotas.labeled.remove_regions(labeled, regions, inplace=True)
    assert np.all(labeled == removed)
    removed = mahotas.labeled.remove_regions(labeled, [])
    assert np.all(labeled == removed)

def test_is_same_labeling():
    np.random.seed(143)
    ell = (np.random.random((256,256))*16).astype(np.intc)
    order = np.arange(1,16)
    np.random.shuffle(order)
    order = np.insert(order, 0, 0)
    assert mh.labeled.is_same_labeling(ell, order[ell])
    ell2 = order[ell]
    ell2[ell == 0] = 1
    
    assert not mh.labeled.is_same_labeling(ell, ell2)

def test_perimeter():
    for r in (40,80,160):
        disk = mh.disk(r, 2)
        p = mahotas.labeled.perimeter(disk)
        p_exact = r*np.pi*2
        assert .9 < (p/p_exact)  < 1.1



def test_remove_regions_where():
    np.random.seed(34)
    for _ in range(4):
        f = np.random.random_sample((128,128)) > .82
        labeled, n = mh.labeled.label(f)
        relabeled = mh.labeled.remove_regions_where(labeled, mh.labeled.labeled_size(labeled) < 2)
        relabeled,_ = mh.labeled.relabel(relabeled)
        sizes = mh.labeled.labeled_size(relabeled)
        assert sizes[1:].min() >= 2

def test_remove_bordering_tuple():
    import mahotas as mh
    import numpy as np
    f = np.zeros((32,32))
    f[0,0] = 1
    f[2,4] = 2
    f.T[4,2] = 3
    f[8,8] = 4
    assert np.any(mh.labeled.remove_bordering(f) == 3)
    assert np.any(mh.labeled.remove_bordering(f, (2,4)) == 4)
    assert np.any(mh.labeled.remove_bordering(f, (2,4)) == 3)
    assert not np.any(mh.labeled.remove_bordering(f, (4,2)) == 3)

def test_as_labeled():

    from mahotas.labeled import _as_labeled
    arr = np.zeros((64,64))
    labeled = np.zeros((64,64), dtype=np.intc)
    funcname = 'testing'

    assert _as_labeled(arr, labeled, funcname, inplace=True) is labeled
    assert _as_labeled(arr, labeled, funcname) is labeled

    lab2 = _as_labeled(arr, labeled, funcname, inplace=False)
    assert lab2 is not labeled
    assert np.all(labeled == lab2)

    assert _as_labeled(arr[::2], labeled[::2], funcname, inplace=False) is not labeled
    assert _as_labeled(arr[::2], labeled[::2], funcname, inplace=None) is not labeled
    assert _as_labeled(arr[::2], labeled[::2], funcname) is not labeled

    @raises(ValueError)
    def t():
        _as_labeled(arr[::2], labeled[::2], funcname, inplace=True)
    t()

    @raises(ValueError)
    def t():
        _as_labeled(arr[::2], labeled, funcname)
    t()


def test_labeled_bbox():
    f = np.random.random((128,128))
    f = f > .8
    f,n = mh.label(f)


    result = mh.labeled.bbox(f)
    result_as = mh.labeled.bbox(f, as_slice=True)
    for _ in range(32):
        ix = np.random.randint(n+1)
        assert np.all(result[ix] == mh.bbox(f == ix))
        assert np.all(result_as[ix] == mh.bbox(f == ix, as_slice=True))

def test_labeled_bbox_zeros():
    'Issue #61'
    def nozeros_test(f):
        result = mh.labeled.bbox(f)
        result_as = mh.labeled.bbox(f, as_slice=True)
        assert not np.all(result == 0)
        for ix in range(4):
            assert np.all(result[ix] == mh.bbox(f == ix))
            assert np.all(result_as[ix] == mh.bbox(f == ix, as_slice=True))

    f = np.array([
        [2,1,1],
        [2,2,1],
        [2,2,3]])
    f3 = np.array([f])
    yield nozeros_test, f
    yield nozeros_test, f3


def test_filter_labeled():

    f = np.random.random(size=(256,256)) > .66
    labeled, nr = mh.label(f)
    no_change,no_nr = mh.labeled.filter_labeled(labeled)
    assert no_nr == nr
    assert np.all(no_change == labeled)

    no_border, border_nr = mh.labeled.filter_labeled(labeled, remove_bordering=True)
    assert nr > border_nr
    assert not np.all(no_border == labeled)

    min_size_3,nr3 = mh.labeled.filter_labeled(labeled, min_size=3)
    assert nr > nr3
    assert mh.labeled.labeled_size(min_size_3).min() == 3

    max_size_3,_ = mh.labeled.filter_labeled(labeled, max_size=3)
    assert mh.labeled.labeled_size(max_size_3)[1:].max() == 3


    all_size_3,_ = mh.labeled.filter_labeled(labeled, max_size=3, min_size=3)
    assert np.all(mh.labeled.labeled_size(all_size_3)[1:] == 3)



# Copyright (C) 2010-2015, Luis Pedro Coelho <luis@luispedro.org>
# vim: set ts=4 sts=4 sw=4 expandtab smartindent:
#
# License: MIT (see COPYING file)


import numpy as np
from . import _convolve
from . import morph
from .internal import _get_output, _normalize_sequence, _verify_is_floatingpoint_type, _as_floating_point_array
from ._filters import mode2int, modes, _check_mode

__all__ = [
    'convolve',
    'convolve1d',
    'daubechies',
    'idaubechies',
    'find',
    'haar',
    'ihaar',
    'median_filter',
    'rank_filter',
    'template_match',
    'gaussian_filter1d',
    'gaussian_filter',
    'wavelet_center',
    'wavelet_decenter',
    'laplacian_2D'
    ]

def convolve(f, weights, mode='reflect', cval=0.0, out=None, output=None):
    '''
    convolved = convolve(f, weights, mode='reflect', cval=0.0, out={new array})

    Convolution of `f` and `weights`

    Convolution is performed in `doubles` to avoid over/underflow, but the
    result is then cast to `f.dtype`.

    Parameters
    ----------
    f : ndarray
        input. Any dimension is supported
    weights : ndarray
        weight filter. If not of the same dtype as `f`, it is cast
    mode : {'reflect' [default], 'nearest', 'wrap', 'mirror', 'constant', 'ignore'}
        How to handle borders
    cval : double, optional
        If `mode` is constant, which constant to use (default: 0.0)
    out : ndarray, optional
        Output array. Must have same shape and dtype as `f` as well as be
        C-contiguous.

    Returns
    -------
    convolved : ndarray of same dtype as `f`
    '''
    if f.dtype != weights.dtype:
        weights = weights.astype(f.dtype)
    if f.ndim != weights.ndim:
        raise ValueError('mahotas.convolve: `f` and `weights` must have the same dimensions')
    output = _get_output(f, out, 'convolve', output=output)
    _check_mode(mode, cval, 'convolve')
    return _convolve.convolve(f, weights, output, mode2int[mode])


def convolve1d(f, weights, axis, mode='reflect', cval=0., out=None):
    '''
    convolved = convolve1d(f, weights, axis, mode='reflect', cval=0.0, out={new array})

    Convolution of `f` and `weights` along axis `axis`.

    Convolution is performed in `doubles` to avoid over/underflow, but the
    result is then cast to `f.dtype`.

    Parameters
    ----------
    f : ndarray
        input. Any dimension is supported
    weights : 1-D ndarray
        weight filter. If not of the same dtype as `f`, it is cast
    axis : int
        Axis along which to convolve
    mode : {'reflect' [default], 'nearest', 'wrap', 'mirror', 'constant', 'ignore'}
        How to handle borders
    cval : double, optional
        If `mode` is constant, which constant to use (default: 0.0)
    out : ndarray, optional
        Output array. Must have same shape and dtype as `f` as well as be
        C-contiguous.

    Returns
    -------
    convolved : ndarray of same dtype as `f`

    See Also
    --------
    convolve : function
        generic convolution
    '''
    weights = np.asanyarray(weights)
    weights = weights.squeeze()
    if weights.ndim != 1:
        raise ValueError('mahotas.convolve1d: only 1-D sequences allowed')
    _check_mode(mode, cval, 'convolve1d')
    if f.flags.contiguous and len(weights) < f.shape[axis]:
        weights = weights.astype(np.double)
        indices = [a for a in range(f.ndim) if a != axis] + [axis]
        rindices = [indices.index(a) for a in range(f.ndim)]
        oshape = f.shape
        f = f.transpose(indices)
        tshape = f.shape
        f = f.reshape((-1, f.shape[-1]))

        out = _get_output(f, out, 'convolve1d')
        _convolve.convolve1d(f, weights, out, mode2int[mode])
        out = out.reshape(tshape)
        out = out.transpose(rindices)
        out = out.reshape(oshape)
        return out
    else:
        index = [None] * f.ndim
        index[axis] = slice(0, None)
        weights = weights[tuple(index)]
        return convolve(f, weights, mode=mode, cval=cval, out=out)


def median_filter(f, Bc=None, mode='reflect', cval=0.0, out=None, output=None):
    '''
    median = median_filter(f, Bc={square}, mode='reflect', cval=0.0, out={np.empty(f.shape, f.dtype})

    Median filter

    Parameters
    ----------
    f : ndarray
        input. Any dimension is supported
    Bc : ndarray or int, optional
        Defines the neighbourhood, default is a square of side 3.
    mode : {'reflect' [default], 'nearest', 'wrap', 'mirror', 'constant', 'ignore'}
        How to handle borders
    cval : double, optional
        If `mode` is constant, which constant to use (default: 0.0)
    out : ndarray, optional
        Output array. Must have same shape and dtype as `f` as well as be
        C-contiguous.

    Returns
    -------
    median : ndarray of same type and shape as ``f``
        median[i,j] is the median value of the points in f close to (i,j)
    '''
    if Bc is None:
        Bc = np.ones((3,) * len(f.shape), f.dtype)
    elif f.dtype != Bc.dtype:
        Bc = Bc.astype(f.dtype)
    if f.ndim != Bc.ndim:
        raise ValueError('mahotas.median_filter: `f` and `Bc` must have the same number of dimensions')
    rank = Bc.sum()//2
    output = _get_output(f, out, 'median_filter', output=output)
    _check_mode(mode, cval, 'median_filter')
    return _convolve.rank_filter(f, Bc, output, int(rank), mode2int[mode])

def mean_filter(f, Bc, mode='ignore', cval=0.0, out=None):
    '''mean = mean_filter(f, Bc, mode='ignore', cval=0.0, out=None)

    Mean filter. The value at ``mean[i,j]`` will be the mean of the values in
    the neighbourhood defined by ``Bc``.

    Parameters
    ----------
    f : ndarray
        input. Any dimension is supported
    Bc : ndarray
        Defines the neighbourhood. Must be explicitly passed, no default.
    mode : {'reflect', 'nearest', 'wrap', 'mirror', 'constant', 'ignore' [ignore]}
        How to handle borders. The default is to ignore points beyond the
        border, so that the means computed near the border include fewer elements.
    cval : double, optional
        If `mode` is constant, which constant to use (default: 0.0)
    out : ndarray, optional
        Output array. Must be a double array with the same shape as `f` as well
        as be C-contiguous.

    Returns
    -------
    mean : ndarray of type double and same shape as ``f``

    See Also
    --------
    median_filter : An alternative filtering method
    '''
    Bc = morph.get_structuring_elem(f, Bc)
    out = _get_output(f, out, 'mean_filter', dtype=np.float64)
    _check_mode(mode, cval, 'mean_filter')
    return _convolve.mean_filter(f, Bc, out, mode2int[mode], cval)


def rank_filter(f, Bc, rank, mode='reflect', cval=0.0, out=None, output=None):
    '''
    ranked = rank_filter(f, Bc, rank, mode='reflect', cval=0.0, out=None)

    Rank filter. The value at ``ranked[i,j]`` will be the ``rank``th largest in
    the neighbourhood defined by ``Bc``.

    Parameters
    ----------
    f : ndarray
        input. Any dimension is supported
    Bc : ndarray
        Defines the neighbourhood. Must be explicitly passed, no default.
    rank : integer
    mode : {'reflect' [default], 'nearest', 'wrap', 'mirror', 'constant', 'ignore'}
        How to handle borders
    cval : double, optional
        If `mode` is constant, which constant to use (default: 0.0)
    out : ndarray, optional
        Output array. Must have same shape and dtype as `f` as well as be
        C-contiguous.

    Returns
    -------
    ranked : ndarray of same type and shape as ``f``
        ranked[i,j] is the ``rank``th value of the points in f close to (i,j)

    See Also
    --------
    median_filter : A special case of rank_filter
    '''
    Bc = morph.get_structuring_elem(f, Bc)
    output = _get_output(f, out, 'rank_filter', output=output)
    _check_mode(mode, cval, 'rank_filter')
    return _convolve.rank_filter(f, Bc, output, rank, mode2int[mode])


def template_match(f, template, mode='reflect', cval=0., out=None, output=None):
    '''Match template to image

    match = template_match(f, template, mode='reflect', cval=0., out={np.empty_like(f)})

    The value at ``match[i,j]`` will be the difference (in squared euclidean
    terms), between `template` and a same sized window on `f` centered on that
    point.

    Note that the computation is performed using the same dtype as ``f``. Thus
    is may overflow if the template is large.

    Parameters
    ----------
    f : ndarray
        input. Any dimension is supported
    template : ndarray
        Template to match. Must be explicitly passed, no default.
    mode : {'reflect' [default], 'nearest', 'wrap', 'mirror', 'constant', 'ignore'}
        How to handle borders
    cval : double, optional
        If `mode` is constant, which constant to use (default: 0.0)
    out : ndarray, optional
        Output array. Must have same shape and dtype as `f` as well as be
        C-contiguous.

    Returns
    -------
    match : ndarray of same type and shape as ``f``
        match[i,j] is the squared euclidean distance between
        ``f[i-s0:i+s0,j-s1:j+s1]`` and ``template`` (for appropriately defined
        ``s0`` and ``s1``).
    '''
    template = template.astype(f.dtype)
    output = _get_output(f, out, 'template_match', output=output)
    _check_mode(mode, cval, 'template_match')
    return _convolve.template_match(f, template, output, mode2int[mode], 0)

def find(f, template):
    '''Match template to image exactly

    coordinates = find(f, template)

    The output is in the same format as the ``np.where`` function.

    Parameters
    ----------
    f : ndarray
        input. Currently, only 2-dimensional images are supported.
    template : ndarray
        Template to match. Must be explicitly passed, no default.

    Returns
    -------
    match : np.array
    coordinates : np.array
        These are the coordinates of the match. The format is similar to the
        output of ``np.where``, but in an ndarray.

    '''
    if f.ndim != 2:
        raise ValueError('mahotas.find: Cannot handle multi-dimensional images')
    template = template.astype(f.dtype)
    out = np.empty(f.shape, bool)
    return _convolve.find2d(f, template, out)


def gaussian_filter1d(array, sigma, axis=-1, order=0, mode='reflect', cval=0., out=None, output=None):
    """
    filtered = gaussian_filter1d(array, sigma, axis=-1, order=0, mode='reflect', cval=0., out={np.empty_like(array)})

    One-dimensional Gaussian filter.

    Parameters
    ----------
    array : ndarray
        input array of a floating-point type

    sigma : float
        standard deviation for Gaussian kernel (in pixel units)
    axis : int, optional
        axis to operate on
    order : {0, 1, 2, 3}, optional
        An order of 0 corresponds to convolution with a Gaussian
        kernel. An order of 1, 2, or 3 corresponds to convolution with
        the first, second or third derivatives of a Gaussian. Higher
        order derivatives are not implemented
    mode : {'reflect' [default], 'nearest', 'wrap', 'mirror', 'constant', 'ignore'}
        How to handle borders
    cval : double, optional
        If `mode` is constant, which constant to use (default: 0.0)
    out : ndarray, optional
        Output array. Must have same shape and dtype as `array` as well as be
        C-contiguous.

    Returns
    -------
    filtered : ndarray
        Filtered version of `array`

    """
    _verify_is_floatingpoint_type(array, 'gaussian_filter1d')
    sigma = float(sigma)
    s2 = sigma*sigma
    # make the length of the filter equal to 4 times the standard
    # deviations:
    lw = int(4.0 * sigma + 0.5)
    if lw <= 0:
        raise ValueError('mahotas.gaussian_filter1d: sigma must be greater or equal to 0.125 [1/8]')
    x = np.arange(2*lw+1, dtype=float)
    x -= lw
    weights = np.exp(x*x/(-2.*s2))
    weights /= np.sum(weights)
    # implement first, second and third order derivatives:
    if order == 0:
        pass
    elif order == 1 : # first derivative
        weights *= -x/s2
    elif order == 2: # second derivative
        weights *= (x*x/s2-1.)/s2
    elif order == 3: # third derivative
        weights *= (3.0 - x*x/s2)*x/(s2*s2)
    else:
        raise ValueError('mahotas.convolve.gaussian_filter1d: Order outside 0..3 not implemented')
    return convolve1d(array, weights, axis, mode, cval, out=output)


def gaussian_filter(array, sigma, order=0, mode='reflect', cval=0., out=None, output=None):
    """
    filtered = gaussian_filter(array, sigma, order=0, mode='reflect', cval=0., out={np.empty_like(array)})

    Multi-dimensional Gaussian filter.

    Parameters
    ----------
    array : ndarray
        input array, any dimension is supported. If the array is an integer
        array, it will be converted to a double array.
    sigma : scalar or sequence of scalars
        standard deviation for Gaussian kernel. The standard
        deviations of the Gaussian filter are given for each axis as a
        sequence, or as a single number, in which case it is equal for
        all axes.
    order : {0, 1, 2, 3} or sequence from same set, optional
        The order of the filter along each axis is given as a sequence
        of integers, or as a single number.  An order of 0 corresponds
        to convolution with a Gaussian kernel. An order of 1, 2, or 3
        corresponds to convolution with the first, second or third
        derivatives of a Gaussian. Higher order derivatives are not
        implemented
    mode : {'reflect' [default], 'nearest', 'wrap', 'mirror', 'constant', 'ignore'}
        How to handle borders
    cval : double, optional
        If `mode` is constant, which constant to use (default: 0.0)
    out : ndarray, optional
        Output array. Must have same shape as `array` as well as be
        C-contiguous. If `array` is an integer array, this must be a double
        array; otherwise, it must have the same type as `array`.

    Returns
    -------
    filtered : ndarray
        Filtered version of `array`

    Notes
    -----
    The multi-dimensional filter is implemented as a sequence of
    one-dimensional convolution filters. The intermediate arrays are
    stored in the same data type as the output. Therefore, for output
    types with a limited precision, the results may be imprecise
    because intermediate results may be stored with insufficient
    precision.
    """
    array = _as_floating_point_array(array)
    output = _get_output(array, out, 'gaussian_filter', output=output)
    orders = _normalize_sequence(array, order, 'gaussian_filter')
    sigmas = _normalize_sequence(array, sigma, 'gaussian_filter')
    output[...] = array[...]
    noutput = None
    for axis in range(array.ndim):
        sigma = sigmas[axis]
        order = orders[axis]
        noutput = gaussian_filter1d(output, sigma, axis, order, mode, cval, noutput)
        output,noutput = noutput,output
    return output

def _wavelet_array(f, inline, func):
    f = _as_floating_point_array(f)
    if f.ndim != 2:
        raise ValueError('mahotas.convolve.%s: Only works for 2D images' % func)
    if not inline:
        return f.copy()
    return f



def _wavelet_center_compute(oshape, border=0, dtype=None, cval=0.0):
    for c in range(1, 16+border):
        nshape = 2**(np.floor(np.log2(oshape))+c)
        nshape = nshape.astype(int)
        delta = nshape - oshape
        delta //= 2
        if np.min(delta) <= border:
            continue
        position = []
        for d,e in zip(delta, oshape):
            position.append( slice(d, d + e) )
        return nshape, position

def wavelet_center(f, border=0, dtype=float, cval=0.0):
    '''
    fc = wavelet_center(f, border=0, dtype=float, cval=0.0)

    ``fc`` is a centered version of ``f`` with a shape that is composed of
    powers of 2.

    Parameters
    ----------
    f : ndarray
        input image
    border : int, optional
        The border to use (default is no border)
    dtype : type, optional
        Type of ``fc``
    cval : float, optional
        Which value to fill the border with (default is 0)

    Returns
    -------
    fc : ndarray

    See Also
    --------
    wavelet_decenter : function
        Reverse function
    '''
    nshape, position = _wavelet_center_compute(f.shape, border)
    nimage = np.zeros(nshape, dtype=dtype)
    nimage += cval
    nimage[position] = f
    return nimage


def wavelet_decenter(w, oshape, border=0):
    '''
    f = wavelet_decenter(w, oshape, border=0)

    Undoes the effect of ``wavelet_center``

    Parameters
    ----------
    w : ndarray
        Wavelet array
    oshape : tuple
        Desired shape
    border : int, optional
        The desired border. This **must** be the same value as was used for
        ``wavelet_center`` 

    Returns
    -------
    f : ndarray
        This will have shape ``oshape``

    See Also
    --------
    wavelet_center : function
        Forward function
    '''
    nshape, position = _wavelet_center_compute(oshape, border)
    return w[position]



def haar(f, preserve_energy=True, inline=False):
    '''
    t = haar(f, preserve_energy=True, inline=False)

    Haar transform

    Parameters
    ----------
    f : 2-D ndarray
        Input image
    preserve_energy : bool, optional
        Whether to normalise the result so that energy is preserved (the
        default).
    inline : bool, optional
        Whether to write the results to the input image. By default, a new
        image is returned. Integer images are always converted to floating
        point and copied.

    See Also
    --------
    ihaar : function
        Reverse Haar transform
    '''
    f = _wavelet_array(f, inline, 'haar')
    _convolve.haar(f)
    _convolve.haar(f.T)
    if preserve_energy:
        f /= 2.0
    return f

_daubechies_codes = [('D%s' % ci) for ci in range(2,21,2)]
def _daubechies_code(c):
    try:
        return _daubechies_codes.index(c)
    except:
        raise ValueError('mahotas.convolve: Known daubechies codes are {0}. You passed in {1}.'.format(_daubechies_codes, c))

def daubechies(f, code, inline=False):
    '''
    filtered = daubechies(f, code, inline=False)

    Daubechies wavelet transform

    This function works best if the image sizes are powers of 2!

    Parameters
    ----------
    f : ndarray
        2-D image
    code : str
        One of 'D2', 'D4', ... 'D20'
    inline : bool, optional
        Whether to write the results to the input image. By default, a new
        image is returned. Integer images are always converted to floating
        point and copied.

    See Also
    --------
    haar : function
        Haar transform (equivalent to D2)
    '''
    f = _wavelet_array(f, inline, 'daubechies')
    code = _daubechies_code(code)
    _convolve.daubechies(f, code)
    _convolve.daubechies(f.T, code)
    return f


def idaubechies(f, code, inline=False):
    '''
    rfiltered = idaubechies(f, code, inline=False)

    Daubechies wavelet inverse transform

    Parameters
    ----------
    f : ndarray
        2-D image
    code : str
        One of 'D2', 'D4', ... 'D20'
    inline : bool, optional
        Whether to write the results to the input image. By default, a new
        image is returned. Integer images are always converted to floating
        point and copied.

    See Also
    --------
    haar : function
        Haar transform (equivalent to D2)
    '''
    f = _wavelet_array(f, inline, 'idaubechies')
    code = _daubechies_code(code)
    _convolve.idaubechies(f.T, code)
    _convolve.idaubechies(f, code)
    return f


def ihaar(f, preserve_energy=True, inline=False):
    '''
    t = ihaar(f, preserve_energy=True, inline=False)

    Reverse Haar transform

    ``ihaar(haar(f))`` is more or less equal to ``f`` (equal, except for
    possible rounding issues).

    Parameters
    ----------
    f : 2-D ndarray
        Input image. If it is an integer image, it is converted to floating
        point (double).
    preserve_energy : bool, optional
        Whether to normalise the result so that energy is preserved (the
        default).
    inline : bool, optional
        Whether to write the results to the input image. By default, a new
        image is returned. Integer images are always converted to floating
        point and copied.

    Returns
    -------
    f : ndarray

    See Also
    --------
    haar : function
        Forward Haar transform
    '''
    f = _wavelet_array(f, inline, 'ihaar')
    _convolve.ihaar(f)
    _convolve.ihaar(f.T)
    if preserve_energy:
        f *= 2.0
    return f

def laplacian_2D(array, alpha = 0.2):
    """
    filtered = laplacian_2D(array, alpha = 0.2)

    2D Laplacian filter.

    Parameters
    ----------
    array : ndarray
        input 2D array. If the array is an integer array, it will be converted 
        to a double array.
    alpha : scalar or sequence of scalars
        controls the shape of Laplacian operator. Must be 0-1. A larger values 
        makes the operator empahsize the diagonal direction.

    Returns
    -------
    filtered : ndarray
        Filtered version of `array`
    """
    array = np.array(array, dtype=np.float)
    if array.ndim != 2:
        raise ValueError('mahotas.laplacian_2D: Only available for 2-dimensional arrays')
        
    alpha = max(0, min(alpha,1));
    ver_hor_weight = (1. - alpha) / (alpha + 1.)
    diag_weight = alpha / (alpha + 1.)
    center = -4. / (alpha + 1.)
    weights = np.array([
    [diag_weight, ver_hor_weight, diag_weight],
    [ver_hor_weight, center, ver_hor_weight],
    [diag_weight, ver_hor_weight, diag_weight]])
    
    output = convolve(array, weights, mode='nearest')
    return output



import sys
import os
import ctypes
import ctypes.util

import numpy as np


_API = {
    'FreeImage_AllocateT': (
        ctypes.c_void_p,
        [ctypes.c_int,  # type
         ctypes.c_int,  # width
         ctypes.c_int,  # height
         ctypes.c_int,  # bpp
         ctypes.c_uint,  # red_mask
         ctypes.c_uint,  # green_mask
         ctypes.c_uint]),  # blue_mask
    'FreeImage_Save': (
        ctypes.c_int,
        [ctypes.c_int,  # type
         ctypes.c_void_p,  # bitmap
         ctypes.c_char_p,  # filename
         ctypes.c_int]),  # flags
    'FreeImage_SetOutputMessage': (
        None,
        [ctypes.c_void_p]),  # callback
    'FreeImage_ConvertToGreyscale': (
        ctypes.c_void_p,  # FIBITMAP * new_bitmap
        [ctypes.c_void_p]),  # FIBITMAP* bitmap
    'FreeImage_GetFIFFromFilename': (
        ctypes.c_int,  # FREE_IMAGE_FORMAT
        [ctypes.c_char_p]),  # const char* filename
    'FreeImage_IsLittleEndian': (
        ctypes.c_int,  # BOOL
        []),
    'FreeImage_FIFSupportsExportBPP': (
        ctypes.c_int,  # BOOL
        [ctypes.c_int,  # FREE_IMAGE_FORMAT format
         ctypes.c_int]),  # int bpp
    'FreeImage_FIFSupportsExportType': (
        ctypes.c_int,  # BOOL
        [ctypes.c_int,  # FREE_IMAGE_FORMAT fif
         ctypes.c_int]),  # FREE_IMAGE_TYPE type
    'FreeImage_Load': (
        ctypes.c_void_p,
        [ctypes.c_int, ctypes.c_char_p, ctypes.c_int]),
    'FreeImage_Unload': (
        None,
        [ctypes.c_void_p]),
    'FreeImage_GetWidth': (
        ctypes.c_uint,
        [ctypes.c_void_p]),
    'FreeImage_GetHeight': (
        ctypes.c_uint,
        [ctypes.c_void_p]),
    'FreeImage_GetImageType': (
        ctypes.c_uint,
        [ctypes.c_void_p]),
    'FreeImage_GetFileTypeFromMemory': (
        ctypes.c_int,
        [ctypes.c_void_p, ctypes.c_int]),
    'FreeImage_GetFileType': (
        ctypes.c_int,
        [ctypes.c_char_p, ctypes.c_int]),
    'FreeImage_GetBPP': (
        ctypes.c_uint,
        [ctypes.c_void_p]),
    'FreeImage_GetPitch': (
        ctypes.c_uint,
        [ctypes.c_void_p]),
    'FreeImage_OpenMultiBitmap': (
        ctypes.c_void_p,  # FIMULTIBITMAP*
        [ctypes.c_int,  # FREE_IMAGE_FORMAT format
         ctypes.c_char_p,  # filename
         ctypes.c_int,  # BOOL create_new
         ctypes.c_int,  # BOOL read_only
         ctypes.c_int,  # BOOL keep_cache_in_memory
         ctypes.c_int]),  # int flags
    'FreeImage_GetPageCount': (
        ctypes.c_int,
        [ctypes.c_void_p]),
    'FreeImage_AppendPage': (
        None,
        [ctypes.c_void_p,  # FIMULTIBITMAP*
         ctypes.c_void_p]),  # BITMAP
    'FreeImage_LockPage': (
        ctypes.c_void_p,  # FIBITMAP*
        [ctypes.c_void_p,  # FIMULTIBITMAP
         ctypes.c_int]),  # int page
    'FreeImage_UnlockPage': (
        None,
        [ctypes.c_void_p,  # FIMULTIBITMAP*
         ctypes.c_void_p,  # FIBITMAP* data
         ctypes.c_int]),  # BOOL changed
    'FreeImage_CloseMultiBitmap': (
        ctypes.c_int,  # BOOL
        [ctypes.c_void_p,  # FIMULTIBITMAP* bitmap
         ctypes.c_int]),  # int flags
    'FreeImage_GetBits': (
        ctypes.c_void_p,
        [ctypes.c_void_p]),
    'FreeImage_OpenMemory': (
        ctypes.c_void_p,
        [ctypes.c_void_p, ctypes.c_uint32]),
    'FreeImage_AcquireMemory': (
        ctypes.c_int,
        [ctypes.c_void_p, ctypes.POINTER(ctypes.c_void_p),
         ctypes.POINTER(ctypes.c_int)]),
    'FreeImage_CloseMemory': (
        None,
        [ctypes.c_void_p]),
    'FreeImage_LoadFromMemory': (
        ctypes.c_void_p,
        [ctypes.c_int, ctypes.c_void_p, ctypes.c_int]),
    'FreeImage_SaveToMemory': (
        ctypes.c_int,
        [ctypes.c_int, ctypes.c_void_p, ctypes.c_void_p, ctypes.c_int]),
    }


class _ctypes_wrapper(object):
    pass


# Albert's ctypes pattern
def _register_api(lib, api):
    nlib = _ctypes_wrapper()
    for f, (restype, argtypes) in api.items():
        try:
            func = getattr(lib, f)
            func.restype = restype
            func.argtypes = argtypes
            setattr(nlib, f, func)
        except Exception:

            def error_raise(*args):
                raise RuntimeError(
                    'mahotas.freeimage: Function `%s` not found in your'
                    ' version of FreeImage. It might be an older version' % f)
            setattr(nlib, f, error_raise)
    return nlib

if sys.platform == 'win32':

    def _load_library(dllname, loadfunction, dllpaths=('', )):
        """Load a DLL via ctypes load function. Return None on failure.

        Try loading the DLL from the current package directory first,
        then from the Windows DLL search path.

        """
        try:
            dllpaths = (os.path.abspath(os.path.dirname(__file__)),
                        ) + dllpaths
        except NameError:
            pass  # no __file__ attribute on PyPy and some frozen distributions
        for path in dllpaths:
            if path:
                # temporarily add the path to the PATH environment variable
                # so Windows can find additional DLL dependencies.
                try:
                    oldenv = os.environ['PATH']
                    os.environ['PATH'] = path + ';' + oldenv
                except KeyError:
                    oldenv = None
            try:
                return loadfunction(os.path.join(path, dllname))
            except (WindowsError, OSError):
                pass
            finally:
                if path and oldenv is not None:
                    os.environ['PATH'] = oldenv
        return None

    _FI = _load_library('FreeImage.dll', ctypes.windll.LoadLibrary)
    if not _FI:
        raise OSError("mahotas.freeimage: could not find FreeImage.dll")

else:
    libname = ctypes.util.find_library('freeimage')
    if libname:
        _FI = ctypes.CDLL(libname)
    else:
        _FI = None
        _lib_dirs = os.environ.get('LD_LIBRARY_PATH', '').split(':')
        _lib_dirs = [_f for _f in _lib_dirs if _f]
        _lib_dirs.extend([
            os.path.dirname(__file__),
            '/lib',
            '/usr/lib',
            '/usr/local/lib',
            '/opt/local/lib',
            ])
        _possible_filenames = (
            'libfreeimage',
            'libFreeImage',
            )
        for d in _lib_dirs:
            for libname in _possible_filenames:
                try:
                    _FI = np.ctypeslib.load_library(libname, d)
                except OSError:
                    pass
                else:
                    break
            if _FI is not None:
                break
    if not _FI:
        raise OSError(
            'mahotas.freeimage: could not find libFreeImage in any of the'
            'following directories: \'%s\'' % '\', \''.join(_lib_dirs))

_FI = _register_api(_FI, _API)

if sys.platform == 'win32':
    _functype = ctypes.WINFUNCTYPE
else:
    _functype = ctypes.CFUNCTYPE


@_functype(None, ctypes.c_int, ctypes.c_char_p)
def _error_handler(fif, message):
    raise RuntimeError('mahotas.freeimage: FreeImage error: %s' % message)

_FI.FreeImage_SetOutputMessage(_error_handler)


class FI_TYPES(object):
    FIT_UNKNOWN = 0
    FIT_BITMAP = 1
    FIT_UINT16 = 2
    FIT_INT16 = 3
    FIT_UINT32 = 4
    FIT_INT32 = 5
    FIT_FLOAT = 6
    FIT_DOUBLE = 7
    FIT_COMPLEX = 8
    FIT_RGB16 = 9
    FIT_RGBA16 = 10
    FIT_RGBF = 11
    FIT_RGBAF = 12

    dtypes = {
        FIT_BITMAP: np.uint8,
        FIT_UINT16: np.uint16,
        FIT_INT16: np.int16,
        FIT_UINT32: np.uint32,
        FIT_INT32: np.int32,
        FIT_FLOAT: np.float32,
        FIT_DOUBLE: np.float64,
        FIT_COMPLEX: np.complex128,
        FIT_RGB16: np.uint16,
        FIT_RGBA16: np.uint16,
        FIT_RGBF: np.float32,
        FIT_RGBAF: np.float32}

    fi_types = {
        (np.uint8, 1): FIT_BITMAP,
        (np.uint8, 3): FIT_BITMAP,
        (np.uint8, 4): FIT_BITMAP,
        (np.uint16, 1): FIT_UINT16,
        (np.int16, 1): FIT_INT16,
        (np.uint32, 1): FIT_UINT32,
        (np.int32, 1): FIT_INT32,
        (np.float32, 1): FIT_FLOAT,
        (np.float64, 1): FIT_DOUBLE,
        (np.complex128, 1): FIT_COMPLEX,
        (np.uint16, 3): FIT_RGB16,
        (np.uint16, 4): FIT_RGBA16,
        (np.float32, 3): FIT_RGBF,
        (np.float32, 4): FIT_RGBAF}

    extra_dims = {
        FIT_UINT16: [],
        FIT_INT16: [],
        FIT_UINT32: [],
        FIT_INT32: [],
        FIT_FLOAT: [],
        FIT_DOUBLE: [],
        FIT_COMPLEX: [],
        FIT_RGB16: [3],
        FIT_RGBA16: [4],
        FIT_RGBF: [3],
        FIT_RGBAF: [4]}

    @classmethod
    def get_type_and_shape(cls, bitmap):
        w = _FI.FreeImage_GetWidth(bitmap)
        h = _FI.FreeImage_GetHeight(bitmap)
        fi_type = _FI.FreeImage_GetImageType(bitmap)
        if not fi_type:
            raise ValueError('mahotas.freeimage: unknown image pixel type')
        dtype = cls.dtypes[fi_type]
        if fi_type == cls.FIT_BITMAP:
            bpp = _FI.FreeImage_GetBPP(bitmap)
            if bpp == 1:
                # This is a special case
                return 'bit', None
            elif bpp == 8:
                extra_dims = []
            elif bpp == 16:
                extra_dims = []
                dtype = np.uint16
            elif bpp == 24:
                extra_dims = [3]
            elif bpp == 32:
                extra_dims = [4]
            else:
                raise ValueError(
                    'mahotas.freeimage: cannot convert %d BPP bitmap' % bpp)
        else:
            extra_dims = cls.extra_dims[fi_type]
        return np.dtype(dtype), extra_dims + [w, h]


class IO_FLAGS(object):
    #Bmp
    BMP_DEFAULT = 0
    BMP_SAVE_RLE = 1

    #Png
    PNG_DEFAULT = 0
    PNG_IGNOREGAMMA = 1

    #Gif
    GIF_DEFAULT = 0
    GIF_LOAD256 = 1
    GIF_PLAYBACK = 2

    #Ico
    ICO_DEFAULT = 0
    ICO_MAKEALPHA = 1

    #Tiff
    TIFF_DEFAULT = 0
    TIFF_CMYK = 0x0001
    TIFF_NONE = 0x0800
    TIFF_PACKBITS = 0x0100
    TIFF_DEFLATE = 0x0200
    TIFF_ADOBE_DEFLATE = 0x0400
    TIFF_CCITTFAX3 = 0x1000
    TIFF_CCITTFAX4 = 0x2000
    TIFF_LZW = 0x4000
    TIFF_JPEG = 0x8000

    #Jpeg
    JPEG_DEFAULT = 0
    JPEG_FAST = 1
    JPEG_ACCURATE = 2
    JPEG_QUALITYSUPERB = 0x80
    JPEG_QUALITYGOOD = 0x100
    JPEG_QUALITYNORMAL = 0x200
    JPEG_QUALITYAVERAGE = 0x400
    JPEG_QUALITYBAD = 0x800
    JPEG_CMYK = 0x1000
    JPEG_PROGRESSIVE = 0x2000

    #Others...
    CUT_DEFAULT = 0
    DDS_DEFAULT = 0
    HDR_DEFAULT = 0
    IFF_DEFAULT = 0
    KOALA_DEFAULT = 0
    LBM_DEFAULT = 0
    MNG_DEFAULT = 0
    PCD_DEFAULT = 0
    PCD_BASE = 1
    PCD_BASEDIV4 = 2
    PCD_BASEDIV16 = 3
    PCX_DEFAULT = 0
    PNM_DEFAULT = 0
    PNM_SAVE_RAW = 0
    PNM_SAVE_ASCII = 1
    PSD_DEFAULT = 0
    RAS_DEFAULT = 0
    TARGA_DEFAULT = 0
    TARGA_LOAD_RGB888 = 1
    WBMP_DEFAULT = 0
    XBM_DEFAULT = 0


class METADATA_MODELS(object):
    FIMD_NODATA = -1
    FIMD_COMMENTS = 0
    FIMD_EXIF_MAIN = 1
    FIMD_EXIF_EXIF = 2
    FIMD_EXIF_GPS = 3
    FIMD_EXIF_MAKERNOTE = 4
    FIMD_EXIF_INTEROP = 5
    FIMD_IPTC = 6
    FIMD_XMP = 7
    FIMD_GEOTIFF = 8
    FIMD_ANIMATION = 9
    FIMD_CUSTOM = 10


class FI_FORMAT(object):
    FIF_UNKNOWN = -1
    FIF_BMP = 0
    FIF_ICO = 1
    FIF_JPEG = 2
    FIF_JNG = 3
    FIF_KOALA = 4
    FIF_LBM = 5
    FIF_IFF = FIF_LBM
    FIF_MNG = 6
    FIF_PBM = 7
    FIF_PBMRAW = 8
    FIF_PCD = 9
    FIF_PCX = 10
    FIF_PGM = 11
    FIF_PGMRAW = 12
    FIF_PNG = 13
    FIF_PPM = 14
    FIF_PPMRAW = 15
    FIF_RAS = 16
    FIF_TARGA = 17
    FIF_TIFF = 18
    FIF_WBMP = 19
    FIF_PSD = 20
    FIF_CUT = 21
    FIF_XBM = 22
    FIF_XPM = 23
    FIF_DDS = 24
    FIF_GIF = 25
    FIF_HDR = 26
    FIF_FAXG3 = 27
    FIF_SGI = 28
    FIF_EXR = 29
    FIF_J2K = 30
    FIF_JP2 = 31
    FIF_PFM = 32
    FIF_PICT = 33
    FIF_RAW = 34


def read(filename, flags=0):
    """Read an image to a numpy array of shape (width, height) for
    greyscale images, or shape (width, height, nchannels) for RGB or
    RGBA images.

    """
    bitmap = _read_bitmap(filename, flags)
    try:
        return _array_from_bitmap(bitmap)
    finally:
        _FI.FreeImage_Unload(bitmap)


def read_multipage(filename, flags=0):
    """Read a multipage image to a list of numpy arrays, where each
    array is of shape (width, height) for greyscale images, or shape
    (nchannels, width, height) for RGB or RGBA images.

    """
    ftype = _FI.FreeImage_GetFileType(_bytestr(filename), 0)
    if ftype == -1:
        raise ValueError(
            'mahotas.freeimage: cannot determine type of file %s'%filename)
    create_new = False
    read_only = True
    keep_cache_in_memory = True
    multibitmap = _FI.FreeImage_OpenMultiBitmap(
        ftype, _bytestr(filename), create_new, read_only, keep_cache_in_memory,
        flags)
    if not multibitmap:
        raise ValueError(
            'mahotas.freeimage: could not open %s'
            ' as multi-page image.' % filename)
    try:
        pages = _FI.FreeImage_GetPageCount(multibitmap)
        arrays = []
        for i in range(pages):
            bitmap = _FI.FreeImage_LockPage(multibitmap, i)
            try:
                arrays.append(_array_from_bitmap(bitmap))
            finally:
                _FI.FreeImage_UnlockPage(multibitmap, bitmap, False)
        return arrays
    finally:
        _FI.FreeImage_CloseMultiBitmap(multibitmap, 0)


def _read_bitmap(filename, flags):
    """Load a file to a FreeImage bitmap pointer"""
    ftype = _FI.FreeImage_GetFileType(_bytestr(filename), 0)
    if ftype == -1:
        raise ValueError(
            'mahotas.freeimage: cannot determine type of file %s' % filename)
    bitmap = _FI.FreeImage_Load(ftype, _bytestr(filename), flags)
    if not bitmap:
        raise ValueError(
            'mahotas.freeimage: could not load file %s' % filename)
    return bitmap


def _wrap_bitmap_bits_in_array(bitmap, shape, dtype):
    """Return an ndarray view on the data in a FreeImage bitmap. Only
    valid for as long as the bitmap is loaded (if single page) / locked
    in memory (if multipage).

    """
    pitch = _FI.FreeImage_GetPitch(bitmap)
    itemsize = dtype.itemsize

    if len(shape) == 3:
        strides = (itemsize, shape[0]*itemsize, pitch)
    else:
        strides = (itemsize, pitch)
    bits = _FI.FreeImage_GetBits(bitmap)

    class DummyArray(object):
        __array_interface__ = {
            'data': (bits, False),
            'strides': strides,
            'typestr': dtype.str,
            'shape': tuple(shape),
            'version': 3,
            }

    return np.array(DummyArray(), copy=False)


def _array_from_bitmap(bitmap):
    """Convert a FreeImage bitmap pointer to a numpy array

    """
    dtype, shape = FI_TYPES.get_type_and_shape(bitmap)
    if type(dtype) == str and dtype == 'bit':
        bitmap8 = _FI.FreeImage_ConvertToGreyscale(bitmap)
        try:
            return _array_from_bitmap(bitmap8).astype(np.bool)
        finally:
            _FI.FreeImage_Unload(bitmap8)
    array = _wrap_bitmap_bits_in_array(bitmap, shape, dtype)

    # swizzle the color components and flip the scanlines to go from
    # FreeImage's BGR[A] and upside-down internal memory format to something
    # more normal
    def n(arr):
        return arr[..., ::-1].T

    if (len(shape) == 3 and _FI.FreeImage_IsLittleEndian() and
        dtype.type == np.uint8):
        b = n(array[0])
        g = n(array[1])
        r = n(array[2])
        if shape[0] == 3:
            return np.dstack((r, g, b))
        elif shape[0] == 4:
            a = n(array[3])
            return np.dstack((r, g, b, a))
        else:
            raise ValueError(
                'mahotas.freeimage: cannot handle images of'
                ' this shape (%s)' % shape)

    # We need to copy because array does *not* own its memory
    # after bitmap is freed.
    return n(array).copy()


def string_tag(bitmap, key, model=METADATA_MODELS.FIMD_EXIF_MAIN):
    """Retrieve the value of a metadata tag with the given string key as a
    string."""
    tag = ctypes.c_int()
    if not _FI.FreeImage_GetMetadata(model, bitmap, _bytestr(key),
                                     ctypes.byref(tag)):
        return
    char_ptr = ctypes.c_char * _FI.FreeImage_GetTagLength(tag)
    return char_ptr.from_address(_FI.FreeImage_GetTagValue(tag)).raw()


def write(array, filename, flags=0):
    """Write a (width, height) or (width, height, nchannels) array to
    a greyscale, RGB, or RGBA image, with file type deduced from the
    filename.

    """
    filename = _bytestr(filename)
    ftype = _FI.FreeImage_GetFIFFromFilename(filename)
    if ftype == -1:
        raise ValueError(
            'mahotas.freeimage: cannot determine type for %s' % filename)
    bitmap, fi_type = _array_to_bitmap(array)
    try:
        if fi_type == FI_TYPES.FIT_BITMAP:
            can_write = _FI.FreeImage_FIFSupportsExportBPP(ftype,
                                      _FI.FreeImage_GetBPP(bitmap))
        else:
            can_write = _FI.FreeImage_FIFSupportsExportType(ftype, fi_type)
        if not can_write:
            raise TypeError(
                'mahotas.freeimage: cannot save image of this type (%s) '
                'to this file type' % array.dtype)
        res = _FI.FreeImage_Save(ftype, bitmap, filename, flags)
        if not res:
            raise RuntimeError(
                'mahotas.freeimage: could not save image properly.')
    finally:
        _FI.FreeImage_Unload(bitmap)


def write_multipage(arrays, filename, flags=0, keep_cache_in_memory=True):
    """Write a list of (width, height) or (nchannels, width, height)
    arrays to a multipage greyscale, RGB, or RGBA image, with file type
    deduced from the filename.

    """
    ftype = _FI.FreeImage_GetFIFFromFilename(_bytestr(filename))
    if ftype == -1:
        raise ValueError(
            'mahotas.freeimage: cannot determine type of file %s' % filename)
    create_new = True
    read_only = False
    multibitmap = _FI.FreeImage_OpenMultiBitmap(
        ftype, _bytestr(filename), create_new, read_only,
        keep_cache_in_memory, 0)
    if not multibitmap:
        raise ValueError(
            'mahotas.freeimage: could not open %s '
            'for writing multi-page image.' % filename)
    try:
        for array in arrays:
            bitmap, _ = _array_to_bitmap(array)
            _FI.FreeImage_AppendPage(multibitmap, bitmap)
            _FI.FreeImage_Unload(bitmap)
    finally:
        _FI.FreeImage_CloseMultiBitmap(multibitmap, flags)


def _array_to_bitmap(array):
    """Allocate a FreeImage bitmap and copy a numpy array into it.

    """
    shape = array.shape
    dtype = array.dtype
    r, c = shape[:2]
    if len(shape) == 2:
        n_channels = 1
        w_shape = (c, r)
    elif len(shape) == 3:
        n_channels = shape[2]
        w_shape = (n_channels, c, r)
    else:
        raise ValueError(
            'mahotas.freeimage: cannot handle image of 4 dimensions')
    try:
        fi_type = FI_TYPES.fi_types[(dtype.type, n_channels)]
    except KeyError:
        raise ValueError(
            'mahotas.freeimage: cannot write arrays of given type and shape.')

    itemsize = array.dtype.itemsize
    bpp = 8 * itemsize * n_channels
    bitmap = _FI.FreeImage_AllocateT(fi_type, c, r, bpp, 0, 0, 0)
    if not bitmap:
        raise RuntimeError(
            'mahotas.freeimage: could not allocate image for storage')
    try:

        def n(arr):  # normalise to freeimage's in-memory format
            return arr.T[:, ::-1]

        wrapped_array = _wrap_bitmap_bits_in_array(bitmap, w_shape, dtype)
        # swizzle the color components and flip the scanlines to go to
        # FreeImage's BGR[A] and upside-down internal memory format
        if (len(shape) == 3 and _FI.FreeImage_IsLittleEndian() and
            dtype.type == np.uint8):
            wrapped_array[0] = n(array[:, :, 2])
            wrapped_array[1] = n(array[:, :, 1])
            wrapped_array[2] = n(array[:, :, 0])
            if shape[2] == 4:
                wrapped_array[3] = n(array[:, :, 3])
        else:
            wrapped_array[:] = n(array)

        return bitmap, fi_type
    except:
        _FI.FreeImage_Unload(bitmap)
        raise


def imsavetoblob(img, filetype, flags=0):
    """
    s = imsavetoblob(img, filetype, flags=0)

    Save `img` to a `str` object

    Parameters
    ----------
    img : ndarray
        input image
    filetype : str or integer
        A file name like string, used only to determine the file type.
        Alternatively, an integer flag (from FI_FORMAT).
    flags : integer, optional

    Returns
    -------
    s : str
        byte representation of `img` in format `filetype`
    """
    if type(filetype) == str:
        ftype = _FI.FreeImage_GetFIFFromFilename(_bytestr(filetype))
    else:
        ftype = filetype
    try:
        bitmap, fi_type = _array_to_bitmap(img)
        mem = _FI.FreeImage_OpenMemory(0, 0)
        if not _FI.FreeImage_SaveToMemory(ftype, bitmap, mem, flags):
            raise IOError(
                'mahotas.freeimage.imsavetoblob: Cannot save to memory.')
        data = ctypes.c_void_p()
        size = ctypes.c_int()
        _FI.FreeImage_AcquireMemory(
            mem, ctypes.byref(data), ctypes.byref(size))
        return ctypes.string_at(data, size)
    finally:
        _FI.FreeImage_CloseMemory(mem)


def imreadfromblob(blob, ftype=None, as_grey=False):
    """
    arr = imreadfromblob(blob, ftype={auto}, as_grey=False)

    Read an image from a blob (string)

    Parameters
    ----------
    blob : str
        Input
    filetype : integer, optional
        input type. By default, infer from image.
    as_grey : boolean, optional
        whether to convert colour images to grey scale

    Returns
    -------
    arr : ndarray
    """
    try:
        mem = _FI.FreeImage_OpenMemory(blob, len(blob))
        if ftype is None:
            ftype = _FI.FreeImage_GetFileTypeFromMemory(mem, 0)
        bitmap = _FI.FreeImage_LoadFromMemory(ftype, mem, 0)
        img = _array_from_bitmap(bitmap)
        if as_grey and len(img.shape) == 3:
            # these are the values that wikipedia says are typical
            transform = np.array([0.30, 0.59, 0.11])
            return np.dot(img, transform)
        return img
    finally:
        _FI.FreeImage_CloseMemory(mem)


def imread(filename, as_grey=False):
    """
    img = imread(filename, as_grey=False)

    Reads an image from file `filename`

    Implementation is based on FreeImage.

    Parameters
    ----------
      filename : file name
      as_grey : Whether to convert to grey scale image (default: no)

    Returns
    -------
      img : ndarray
    """
    img = read(filename)
    if as_grey and len(img.shape) == 3:
        # these are the values that wikipedia says are typical
        transform = np.array([0.30, 0.59, 0.11])
        return np.dot(img, transform)
    return img


def imsave(filename, img):
    """
    imsave(filename, img)

    Save image to disk

    Image type is inferred from filename

    Implementation is based on FreeImage.

    Parameters
    ----------
      filename : file name
      img : image to be saved as nd array
    """
    write(img, filename)


if sys.version_info[0] > 2:
    import locale
    _, _encoding = locale.getdefaultlocale()
    if _encoding is None:
        _encoding = 'UTF-8'
    _bytestr = lambda x: x.encode(_encoding)
else:
    _bytestr = str



#!/opt/anaconda1anaconda2anaconda3\python.exe

import sys
from os import path
import numpy as np
import mahotas as mh
import argparse
from itertools import chain

RED = '\033[91m'
GREEN = '\033[92m'
YELLOW = '\033[93m'
LIGHT_PURPLE = '\033[94m'
PURPLE = '\033[95m'
END = '\033[0m'

def print_error(text, color=True):
    '''Prints error message

    Arguments
    ---------
    text : str
        Error message
    color : bool, optional
        Whether to print in colour.
    '''
    if color and sys.stderr.isatty():
        sys.stderr.write("{}ERROR: {}{}\n".format(RED, text, END))
    else:
        sys.stderr.write("ERROR: {}\n".format(text))


def read_bw(fname, options):
    '''Read image `fname` as greyscale

    Parameters
    ----------
    fname : str, file-name
    options : argparse result

    Returns
    -------
    image : ndarray
        Two dimensional ndarray
    '''
    im = mh.imread(fname)
    if im.ndim == 2:
        return im
    if im.ndim == 3:
        if options.convert_to_bw == 'max' or im.ptp(2).max() == 0:
            # This is a greyscale image, saved as colour
            return im.max(2)
        if options.convert_to_bw == 'yes':
            return mh.colors.rgb2grey(im, dtype=np.uint8)
    print_error("{} is not a greyscale image (and --convert-to-bw was not specified)".format(fname), not options.no_color)
    sys.exit(1)

def _write_row(output, items, header=None):
    if header:
        output.write(header)
    for it in items:
        output.write('\t')
        output.write(str(it))
    output.write('\n')

def main():
    sys.stderr.write(mh.citation(print_out=False, short=True))
    sys.stderr.write('\n\n')
    parser = argparse.ArgumentParser(
            description='Compute features using mahotas')
    parser.add_argument(
                    'fnames', metavar='input_file_name', nargs='+', type=str,
                            help='Image files names')
    parser.add_argument(
                    '--output', default='features.tsv', type=str,
                            help='Output file for feature files')
    parser.add_argument(
                    '--clobber', default=False, action='store_true',
                            help='Overwrite output file (if it exists)')
    parser.add_argument(
                    '--convert-to-bw', default='no',
                    help='Convert color images to greyscale.\nAcceptable values:\n\tno: raises an error (default)' +
                        '\n\tmax: use max projection' +
                        '\n\tyes: use rgb2gray')
    parser.add_argument(
                    '--no-color', default=False, action='store_true',
                            help='Do not print in color (for error and warning messages)')
    parser.add_argument(
                    '--haralick', default=False, action='store_true',
                            help='Compute Haralick features')
    parser.add_argument(
                    '--lbp', default=False, action='store_true',
                            help='Compute LBP (linear binary patterns) features')
    parser.add_argument(
                    '--lbp-radius', default=8, action='store', type=int,
                            help='Radius to use for LBP features')
    parser.add_argument(
                    '--lbp-points', default=6, action='store', type=int,
                            help='Nr of points to use for LBP features')
    args = parser.parse_args()
    if not (args.haralick or args.lbp):
        sys.stderr.write('''\
No features selected. Doing nothing.

For example, use --haralick switch to compute Haralick features\n''')
        sys.exit(1)

    if not args.clobber and path.exists(args.output):
        print_error('Output file ({}) already exists. Refusing to overwrite results without --clobber argument.'.format(args.output))
        sys.exit(2)

    output = open(args.output, 'w')
    colnames = []
    if args.haralick:
        hlabels = mh.features.texture.haralick_labels[:-1]
        colnames.extend(["mean:{}".format(ell) for ell in hlabels])
        colnames.extend(["ptp:{}".format(ell) for ell in hlabels])
    if args.lbp:
        from mahotas.features.lbp import lbp_names
        colnames.extend(lbp_names(args.lbp_radius, args.lbp_points))
    _write_row(output, colnames)

    for fname in args.fnames:
        cur = []
        im = read_bw(fname, args)
        if args.haralick:
            har = mh.features.haralick(im, return_mean_ptp=True)
            cur.append(har)
        if args.lbp:
            cur.append(mh.features.lbp(im, args.lbp_radius, args.lbp_points))

        _write_row(output, chain.from_iterable(cur), fname)
    output.close()

if __name__ == '__main__':
    main()



import numpy as np
import mahotas as mh
from mahotas.features.moments import moments

def _slow(A, p0, p1, cm):
    c0,c1 = cm
    I,J = np.meshgrid(np.arange(A.shape[0],dtype=float), np.arange(A.shape[1], dtype=float))
    I -= c0
    J -= c1
    I **= p0
    J **= p1
    return (I.T * J.T * A).sum()

def test_smoke():
    assert moments(np.zeros((100,23)), 2, 2) == 0.0
    assert moments(np.ones((100,23)), 2, 2) != 0.0
    assert moments(np.ones((100,23)), 0, 0) == 100*23
    assert moments(np.ones((100,23)), 2, 2) != moments(np.ones((100,23)), 2, 2, cm=(50,12))

def test_against_slow():
    def perform(p0, p1, cm, A):
        assert moments(A, p0, p1, cm) == _slow(A, p0,p1,cm)

    A = (np.arange(2048) % 14).reshape((32, -1))
    yield perform, 2, 2, (22, 22), A
    yield perform, 2, 2, (20, 22), A
    yield perform, 2, 2, (0, 0), A
    yield perform, 1, 2, (0, 0), A
    yield perform, 1, 0, (0, 0), A


def test_normalize():
    A,B = np.meshgrid(np.arange(128),np.arange(128))
    for p0,p1 in [(1,1), (1,2), (2,1), (2,2)]:
        def f(im):
            return moments(im, p0, p1, cm=mh.center_of_mass(im), normalize=1)
        im = A+B
        fs = [f(im), f(im[::2]), f(im[:,::2]), f(im[::2, ::2])]
        assert np.var(fs) < np.mean(np.abs(fs))/10

def test_moments01():
    im = np.zeros((16,16))
    im += np.arange(16)
    im -= im.mean()
    assert np.abs(mh.moments(im , 1, 0)) < 0.1

def test_expression():
    import mahotas as mh
    import numpy as np
    im = (np.zeros((12,16)) + np.arange(16))
    im -= im.mean()
    c0,c1 = 4,7
    p0,p1 = 2, 3
    for p0,p1 in [(0,1),
                    (1,0),
                    (1,2),
                    (3,2),
                    (2,0)]:
        sum_r = sum((im[i,j] * (i - c0)**p0 * (j - c1)**p1) for i in range(im.shape[0]) for j in range(im.shape[1]))
        mm_r = mh.moments(im , p0, p1, cm=(c0,c1))
        assert np.abs(mm_r - sum_r) < 1.



import mahotas.features.shape
import numpy as np
import mahotas as mh
from mahotas.features.shape import roundness, eccentricity

def test_eccentricity():
    D = mh.disk(32, 2)
    ecc = mahotas.features.shape.eccentricity(D)
    assert 0 <= ecc < .01

    Index = np.indices((33,33)).astype(float)
    Index -= 15
    X,Y = Index
    ellipse = ((X**2+2*Y**2) < 12**2)
    assert 0 < mahotas.features.shape.eccentricity(ellipse) < 1

def test_roundness():
    Y,X = -24 + np.indices((48,48)).astype(float)
    r = roundness( (Y ** 2. + X**2.) < 4**2. )
    assert r > 0
    r2 = roundness( (Y ** 2. + 2* X**2.) < 4**2. )
    assert r2 > 0
    assert r2 < r

def test_zeros():
    assert roundness(np.zeros((10,10))) == 0
    assert eccentricity(np.zeros((10,10))) == 0
    I = np.zeros((16,16))
    I[8:4:12] = 1
    assert eccentricity(I) == 0

def test_ellipse_axes():
    Y,X = np.mgrid[:1024,:1024]
    Y = Y/1024.
    X = X/1024.
    im = ((2.*(Y - .5)**2 + (X - .5)**2) < .2)
    maj,min = mh.features.ellipse_axes(im)
    assert np.abs(2 - (maj/min)**2) < .01

    maj2,min2 = mh.features.ellipse_axes(im.T)

    assert np.abs(maj - maj2) < .001
    assert np.abs(min - min2) < .001

    im = (((Y - .5)**2 + (X - .5)**2) < .2)
    maj,min = mh.features.ellipse_axes(im)
    assert np.abs(1 - (maj/min)**2) < .01



import numpy as np
import mahotas as mh
from PIL import Image

def imread(filename, as_grey=False):
    '''Read an image into a ndarray from a file.

    This function depends on PIL (or Pillow) being installed.

    Parameters
    ----------
    filename : str
        filename
    as_grey : boolean, optional
        Whether to convert to grey scale image (default: no)
    '''
    im = Image.open(filename)
    array = np.array(im)
    if as_grey and array.ndim != 2:
        array = mh.colors.rgb2grey(array)
    return array

def imsave(filename, array):
    '''
    Writes `array` into file `filename`

    This function depends on PIL (or Pillow) being installed.

    Parameters
    ----------
    filename : str
        path on file system
    array : ndarray-like
    '''
    im = Image.fromarray(array)
    im.save(filename)




from mahotas import _filters
from nose.tools import raises

@raises(ValueError)
def test_bad_mode():
    _filters._check_mode('nayrest', 0., 'f')

@raises(NotImplementedError)
def test_cval_not_zero():
    _filters._check_mode('constant', 1.2, 'f')

def test_good_mode():
    for mode in _filters.modes:
        _filters._check_mode(mode, 0., 'f')




from __future__ import print_function
import numpy as np
import mahotas as mh
from mahotas.features import surf
from pylab import *

from os import path

f = mh.demos.load('luispedro', as_grey=True)
f = f.astype(np.uint8)
spoints = surf.surf(f, 4, 6, 2)
print("Nr points:", len(spoints))

try:
    import milk
    descrs = spoints[:,5:]
    k = 5
    values, _  =milk.kmeans(descrs, k)
    colors = np.array([(255-52*i,25+52*i,37**i % 101) for i in range(k)])
except:
    values = np.zeros(100)
    colors = np.array([(255,0,0)])

f2 = surf.show_surf(f, spoints[:100], values, colors)
imshow(f2)
show()



import numpy as np
from mahotas.euler import euler, _euler_lookup4, _euler_lookup8
from nose.tools import raises

def test_lookup():
    Q1 = [np.array(q, np.bool) for q in ([[0,0],[1,0]], [[0,0],[0,1]], [[0,1],[0,0]], [[1,0],[0,0]]) ]
    Q2 =  [(~q) for q in Q1]
    Q3 = [np.array(q, np.bool) for q in ([[0,1],[1,0]], [[1,0],[0,1]]) ]

    def _value(q, lookup):
        q = q.ravel()
        value = np.dot(q, (1,2,4,8))
        return lookup[value]

    for q in Q1:
        assert _value(q, _euler_lookup8) == .25
        assert _value(q, _euler_lookup4) == .25
    for q in Q2:
        assert _value(q, _euler_lookup8) == -.25
        assert _value(q, _euler_lookup4) == -.25
    for q in Q3:
        assert _value(q, _euler_lookup8) == -.5
        assert _value(q, _euler_lookup4) ==  .5

def test_euler():
    f = np.zeros((16,16), np.bool)
    f[4:8,4:8] = 1
    assert euler(f) == 1
    assert euler(f, 4) == 1

    f[6:7,5:7] = 0

    assert euler(f) == 0
    assert euler(f, 4) == 0

@raises(ValueError)
def test_euler7():
    f = np.arange(100)
    f = (f % 5) == 1
    f = f.reshape((10,10))
    euler(f, 7)




# Copyright (C) 2008-2015, Luis Pedro Coelho <luis@luispedro.org>
# vim: set ts=4 sts=4 sw=4 expandtab smartindent:
#
# LICENSE: MIT

from __future__ import division
import numpy as np
from .morph import get_structuring_elem
from . import _labeled
from .internal import _get_output
from ._filters import _checked_mode2int
import mahotas as mh

__all__ = [
    'bbox',
    'borders',
    'border',
    'bwperim',
    'filter_labeled',
    'label',
    'labeled_sum',
    'labeled_max',
    'labeled_size',
    'relabel',
    'is_same_labeling',
    'perimeter',
    'remove_bordering',
    'remove_regions',
    'remove_regions_where',
    ]

def label(array, Bc=None, out=None, output=None):
    '''
    labeled, nr_objects = label(array, Bc={3x3 cross}, output={new array})

    Label the array, which is interpreted as a binary array

    This is also called *connected component labeled*, where the connectivity
    is defined by the structuring element ``Bc``.

    See: http://en.wikipedia.org/wiki/Connected-component_labeling

    Parameters
    ----------
    array : ndarray
        This will be interpreted as binary array
    Bc : ndarray, optional
        This is the structuring element to use
    out : ndarray, optional
        Output array. Must be a C-array, of type np.int32

    Returns
    -------
    labeled : ndarray
        Labeled result
    nr_objects : int
        Number of objects
    '''
    output = _get_output(array, out, 'labeled.label', np.int32, output=output)
    output[:] = (array != 0)
    Bc = get_structuring_elem(output, Bc)
    nr_objects = _labeled.label(output, Bc)
    return output, nr_objects

def relabel(labeled, inplace=False):
    '''
    relabeled, nr_objs = relabel(labeled, inplace=False)

    Relabeling ensures that ``relabeled`` is a labeled image such that every
    label from 1 to ``relabeled.max()`` is used (0 is reserved for the
    background and is passed through).

    Example::

        labeled,n = label(some_binary_map)
        for region in xrange(n):
            if not good_region(labeled, region + 1):
                # This deletes the region:
                labeled[labeled == (region + 1)] = 0
        relabel(labeled, inplace=True)

    Parameters
    ----------
    relabeled : ndarray of int
        A labeled array
    inplace : boolean, optional
        Whether to perform relabeling inplace, erasing the values in
        ``labeled`` (default: False)

    Returns
    -------
    relabeled: ndarray
    nr_objs : int
        Number of objects

    See Also
    --------
    label : function
    '''
    labeled = _as_labeled(labeled, labeled, 'relabel', inplace=inplace)
    n = _labeled.relabel(labeled)
    return labeled, n

def is_same_labeling(labeled0, labeled1):
    '''
    same = is_same_labeling(labeled0, labeled1)

    Checks whether ``labeled0`` and ``labeled1`` represent the same labeling
    (i.e., whether they are the same except for a possible change of label
    values).

    Note that the background (value 0) is treated differently. Namely

    is_same_labeling(a, b) implies np.all( (a == 0) == (b == 0) )

    Parameters
    ----------
    labeled0 : ndarray of int
        A labeled array
    labeled1 : ndarray of int
        A labeled array

    Returns
    -------
    same : bool
        True if the labelings passed as argument are equivalent

    See Also
    --------
    label : function
    relabel : function
    '''
    labeled0 = _convert_labeled(labeled0)
    labeled1 = _convert_labeled(labeled1)
    return _labeled.is_same_labeling(labeled0, labeled1)


def remove_regions(labeled, regions, inplace=False):
    '''
    removed = remove_regions(labeled, regions, inplace=False):

    Removes the regions in ``regions``. If an elementwise ``in`` operator
    existed, this would be equivalent to the following::

        labeled[ labeled element-wise-in regions ] = 0

    This function **does not** relabel its arguments. You can use the
    ``relabel`` function for that::

        removed = relabel(remove_regions(labeled, regions))

    Or, saving one image allocation::

        removed = relabel(remove_regions(labeled, regions), inplace=True)

    This is the same, but reuses the memory in the relabeling operation.

    Parameters
    ----------
    relabeled : ndarray of int
        A labeled array
    regions : sequence of int
        These regions will be removed
    inplace : boolean, optional
        Whether to perform removal inplace, erasing the values in
        ``labeled`` (default: False)

    Returns
    -------
    removed : ndarray

    See Also
    --------
    relabel : function
        After removing unecessary regions, it is often a good idea to relabel
        your label image.
    '''
    labeled = _as_labeled(labeled, labeled, 'remove_regions', inplace=inplace)
    regions = np.asarray(regions, dtype=np.intc)
    regions = np.unique(regions)
    _labeled.remove_regions(labeled, regions)
    return labeled


def remove_regions_where(labeled, conditions, inplace=False):
    '''Remove regions based on a boolean array

    A region is removed if ``conditions[region-id]`` evaluates true.

    This function **does not** relabel its arguments. You can use the
    ``relabel`` function for that::

        removed = relabel(remove_regions_where(labeled, conditions))

    Or, saving one image allocation::

        removed = relabel(remove_regions(labeled, conditions), inplace=True)

    This is the same, but reuses the memory in the relabeling operation.


    See Also
    --------
    remove_regions : function
        Variation of this function which uses integer indexing
    '''
    regions, = np.where(conditions)
    return remove_regions(labeled, regions, inplace=inplace)


def remove_bordering(labeled, rsize=1, out=None, output=None):
    '''
    slabeled = remove_bordering(labeled, rsize=1, out={np.empty_like(im)})

    Remove objects that are touching the border.

    Pass ``labeled`` as ``out`` to achieve in-place operation.

    Parameters
    ----------
    labeled : ndarray
        Labeled array
    rsize : int or tuple, optional
        Minimum distance to the border (in Manhatan distance) to allow an
        object to survive. May be int or tuple with len == labeled.ndim.
    out : ndarray, optional
        If ``im`` is passed as ``out``, then it operates inline.

    Returns
    -------
    slabeled : ndarray
        Subset of ``labeled``
    '''
    im = labeled
    invalid = set()
    index = [slice(None,None,None) for _ in range(im.ndim)]
    if type(rsize) is not tuple:
        rsize = (rsize,)*im.ndim
    for dim in range(im.ndim):
        for bordering in (
                    slice(rsize[dim]),
                    slice(im.shape[dim]-rsize[dim], None)
                        ):
            index[dim] = bordering
            for val in np.unique(im[tuple(index)].ravel()):
                if val != 0:
                    invalid.add(val)
        index[dim] = slice(None,None,None)
    if out is None and output is not None: #pragma: no cover
        import warnings
        warnings.warn('Using deprecated `output` argument in function `%s`. Please use `out` in the future.' % 'remove_bordering', DeprecationWarning)
        out = output
    if out is None:
        out = im.copy()
    elif out is not im:
        out[:] = im
    for val in invalid:
        out *= (im != val)
    return out


def border(labeled, i, j, Bc=None, out=None, always_return=True, output=None):
    '''
    border_img = border(labeled, i, j, Bc={3x3 cross}, out={np.zeros(labeled.shape, bool)}, always_return=True)

    Compute the border region between `i` and `j` regions.

    A pixel is on the border if it has value `i` (or `j`) and a pixel in its
    neighbourhood (defined by `Bc`) has value `j` (or `i`).

    Parameters
    ----------
    labeled : ndarray of integer type
        input labeled array
    i : integer
    j : integer
    Bc : structure element, optional
    out : ndarray of same shape as `labeled`, dtype=bool, optional
        where to store the output. If ``None``, a new array is allocated
    always_return : bool, optional
        if false, then, in the case where there is no pixel on the border,
        returns ``None``. Otherwise (the default), it always returns an array
        even if it is empty.

    Returns
    -------
    border_img : boolean ndarray
        Pixels are True exactly where there is a border between `i` and `j` in `labeled`
    '''
    Bc = get_structuring_elem(labeled, Bc)
    output = _get_output(labeled, out, 'labeled.border', bool, output=output)
    output.fill(False)
    return _labeled.border(labeled, Bc, output, i, j, bool(always_return))

def borders(labeled, Bc=None, out=None, output=None, mode='constant'):
    '''
    border_img = borders(labeled, Bc={3x3 cross}, out={np.zeros(labeled.shape, bool)})

    Compute border pixels

    A pixel is on a border if it has value `i` and a pixel in its neighbourhood
    (defined by `Bc`) has value `j`, with ``i != j``.

    Parameters
    ----------
    labeled : ndarray of integer type
        input labeled array
    Bc : structure element, optional
    out : ndarray of same shape as `labeled`, dtype=bool, optional
        where to store the output. If ``None``, a new array is allocated
    mode : {'reflect', 'nearest', 'wrap', 'mirror', 'constant' [default], 'ignore'}
        How to handle borders

    Returns
    -------
    border_img : boolean ndarray
        Pixels are True exactly where there is a border in `labeled`
    '''
    Bc = get_structuring_elem(labeled, Bc)
    output = _get_output(labeled, out, 'labeled.borders', bool, output=output)
    output.fill(False)
    return _labeled.borders(labeled, Bc, output, _checked_mode2int(mode, 0.0, 'borders'))


def bwperim(bw, n=4, mode="constant"):
    '''
    perim = bwperim(bw, n=4)

    Find the perimeter of objects in binary images.

    A pixel is part of an object perimeter if its value is one and there
    is at least one zero-valued pixel in its neighborhood.

    By default the neighborhood of a pixel is 4 nearest pixels, but
    if `n` is set to 8 the 8 nearest pixels will be considered.

    Parameters
    ----------
    bw : ndarray
        A black-and-white image (any other image will be converted to black & white)
    n : int, optional
        Connectivity. Must be 4 or 8 (default: 4)
    mode : {'reflect', 'nearest', 'wrap', 'mirror', 'constant' [default], 'ignore'}
        How to handle borders

    Returns
    -------
    perim : ndarray
        A boolean image

    See Also
    --------
    borders : function
        This is a more generic function
    '''
    bw = (bw != 0)
    return bw&borders(bw, n, mode=mode)

def _as_labeled(array, labeled, funcname, inplace='unused'):
    '''

    labeled = _as_labeled(array, labeled, funcname, inplace='unused')

    Parameters
    ----------
    array : ndarray
    labeled : ndarray
    funcname : str
    inplace : bool or str, optional
        handles ``inplace`` arguments
    '''
    if inplace == 'unused':
        labeled = np.require(labeled, dtype=np.intc, requirements="CW")
    elif not inplace:
        labeled = np.array(labeled, dtype=np.intc)
    elif labeled.dtype != np.intc or not labeled.flags.carray:
        raise ValueError('mahotas.labeled.%s: labeled must be a C-array of type int' % funcname)

    if array.shape != labeled.shape:
        raise ValueError('mahotas.labeled.%s: `array` is not the same size as `labeled`' % funcname)
    return labeled


def _convert_labeled(labeled):
    return np.require(labeled, dtype=np.intc, requirements="CW")

def labeled_sum(array, labeled, minlength=None):
    '''
    sums = labeled_sum(array, labeled, minlength=None)

    Labeled sum. sum will be an array of size ``labeled.max() + 1``, where
    ``sum[i]`` is equal to ``np.sum(array[labeled == i])``.

    Parameters
    ----------
    array : ndarray of any type
    labeled : int ndarray
        Label map. This is the same type as returned from ``mahotas.label()``
    minlength : int, optional
        Minimum size of return array. If labeled has fewer than ``minlength``
        regions, 0s are added to the result.  (optional)

    Returns
    -------
    sums : 1-d ndarray of ``array.dtype``
    '''
    labeled = _as_labeled(array, labeled, 'labeled_sum')
    maxv = labeled.max() + 1
    if minlength is not None:
        maxv = max(maxv, minlength)
    output = np.empty(maxv, dtype=array.dtype)
    _labeled.labeled_sum(array, labeled, output)
    return output


def labeled_max(array, labeled):
    '''
    mins = labeled_max(array, labeled, minlength=None)

    Labeled minimum. ``mins`` will be an array of size ``labeled.max() + 1``, where
    ``mins[i]`` is equal to ``np.min(array[labeled == i])``.

    Parameters
    ----------
    array : ndarray of any type
    labeled : int ndarray
        Label map. This is the same type as returned from ``mahotas.label()``

    Returns
    -------
    mins : 1-d ndarray of ``array.dtype``
    '''
    labeled = _as_labeled(array, labeled, 'labeled_max')
    maxv = labeled.max() + 1
    output = np.empty(maxv, dtype=array.dtype)
    _labeled.labeled_max_min(array, labeled, output, True)
    return output


def labeled_min(array, labeled):
    '''
    maxs = labeled_min(array, labeled)

    Labeled maximum. ``maxs`` will be an array of size ``labeled.max() + 1``, where
    ``maxs[i]`` is equal to ``np.max(array[labeled == i])``.

    Parameters
    ----------
    array : ndarray of any type
    labeled : int ndarray
        Label map. This is the same type as returned from ``mahotas.label()``

    Returns
    -------
    maxs : 1-d ndarray of ``array.dtype``
    '''
    labeled = _as_labeled(array, labeled, 'labeled_min')
    maxv = labeled.max() + 1
    output = np.empty(maxv, dtype=array.dtype)
    _labeled.labeled_max_min(array, labeled, output, False)
    return output

def labeled_size(labeled):
    '''
    sizes = labeled_size(labeled)

    Equivalent to::

        for i in range(...):
            sizes[i] = np.sum(labeled == i)

    but, naturally, much faster.

    Parameters
    ----------
    labeled : int ndarray

    Returns
    -------
    sizes : 1-d ndarray of int

    See Also
    --------
    mahotas.fullhistogram : almost same function by another name (the only
    difference is that that function only accepts unsigned integer types).
    '''
    from .histogram import fullhistogram
    return fullhistogram(labeled.astype(np.uint32))


_perimeter_magic = np.array([
                [10, 2, 10],
                [ 2, 1,  2],
                [10, 2, 10]], np.uint8)
_perimeter_values = None

# This implementation was adapted from scikit-image's implementation
def perimeter(bwimage, n=4, mode="constant"):
    """
    p = perimeter(bwimage, n=4, mode="constant")

    Calculate total perimeter of all objects in binary image.

    Parameters
    ----------
    bwimage : array
        binary image
    n : int, optional
        passed to ``bwperim`` as is
    mode : str, optional
        passed to ``bwperim`` as is

    Returns
    -------
    p : float
        total perimeter of all objects in binary image

    See Also
    --------
    bwperim : function
        Finds the perimeter region

    References
    ----------
    .. [1] K. Benkrid, D. Crookes. Design and FPGA Implementation of
           a Perimeter Estimator. The Queen's University of Belfast.
           http://www.cs.qub.ac.uk/~d.crookes/webpubs/papers/perimeter.doc
    """
    global _perimeter_values
    perim = bwperim(bwimage, n, mode)
    perim = perim.astype(np.uint8)

    histogram = mh.fullhistogram(
                    mh.convolve(perim, _perimeter_magic))

    if _perimeter_values is None:
        _perimeter_values = np.zeros(34, float)
        _perimeter_values[[5, 7, 15, 17, 25, 27]] = 1
        _perimeter_values[[21, 33]] = np.sqrt(2)
        _perimeter_values[[13, 23]] = (1 + np.sqrt(2)) / 2

    size = min(34, len(histogram))
    return np.dot(histogram[:size], _perimeter_values[:size])


def bbox(f, as_slice=False):
    '''
    Bounding boxes of all objects in a labeled array.

    After::

        bboxes = mh.labeled.bbox(f)

    ``bboxes[34]`` will contain the bounding box of ``(f == 34)``.

    Parameters
    --------
    f : integer ndarray
    as_slice : boolean, optional
        Whether to return slice objects instead of integer coordinates
        (default: False).

    Returns
    -------
    bboxes : ndarray

    See Also
    --------
    mh.bbox : the binary version of this function
    '''
    import mahotas._bbox
    n = f.max()
    output = np.empty( f.ndim * 2 * (n+1), np.intp)
    output = mahotas._bbox.bbox_labeled(f, output)
    output = output.reshape((n+1, 2*f.ndim))
    if as_slice:
        output = [tuple([slice(s,e) for s,e in r.reshape((-1,2))]) for r in output]
    return output

def filter_labeled(labeled, remove_bordering=False, min_size=None, max_size=None):
    '''Filter labeled regions based on a series of conditions

    .. versionadded :: 1.4.1

    Parameters
    ----------
    labeled : labeled array
    remove_bordering : bool, optional
        whether to remove regions that touch the border
    min_size : int, optional
        Minimum size (in pixels) of objects to keep (default is no minimum)
    max_size : int, optional
        Maximum size (in pixels) of objects to keep (default is no maximum)

    Returns
    -------
    filtered : labeled array
    nr : int
        number of new labels
    '''
    from mahotas.labeled import remove_regions, labeled_size
    import mahotas as mh
    labeled = _as_labeled(labeled, labeled, 'filter_labeled')
    if remove_bordering:
        labeled = mh.labeled.remove_bordering(labeled)
        labeled,nr = mh.labeled.relabel(labeled)
    else:
        nr = labeled.max()

    to_keep = np.ones(nr+1, bool)

    if min_size is not None or max_size is not None:
        sizes = labeled_size(labeled)
        if min_size:
            to_keep &= (sizes >= min_size)
        if max_size:
            to_keep &= (sizes <= max_size)
    to_keep[0] = True
    to_remove = np.where(~to_keep)
    labeled = remove_regions(labeled, to_remove)
    labeled,nr = mh.labeled.relabel(labeled, inplace=True)
    return labeled, nr



import numpy as np
from mahotas.features import texture
import mahotas as mh
import mahotas.features._texture
from nose.tools import raises

def test__cooccurence():
    cooccurence = mahotas.features._texture.cooccurence
    f = np.array([
          [0,1,1,1],
          [0,0,1,1],
          [2,2,2,2],
        ])
    Bc = np.zeros((3,3), f.dtype)
    Bc[1,2] = 1
    res = np.zeros((5,5), np.int32)
    cooccurence(f, res, Bc, 0)
    assert res[0,0] == 1
    assert res[0,1] == 2
    assert res[1,0] == 0
    assert res[1,1] == 3
    assert res[2,2] == 3
    assert not np.any(res[2,:2])
    assert not np.any(res[:2,2])
    res[:3,:3] = 0
    assert not np.any(res)

    res = np.zeros((5,5), np.int32)
    Bc = np.zeros((3,3), f.dtype)
    Bc[2,2] = 1
    cooccurence(f, res, Bc, 0)
    assert res[0,0] == 1
    assert res[0,1] == 0
    assert res[0,2] == 2
    assert res[1,0] == 0
    assert res[1,1] == 2
    assert res[1,2] == 1
    res[:3,:3] = 0
    assert not np.any(res)

def test_cooccurence_errors():
    f2 = np.zeros((6,6), np.uint8)
    f3 = np.zeros((6,6,6), np.uint8)
    f4 = np.zeros((6,6,6,6), np.uint8)
    @raises(ValueError)
    def c_1():
        texture.cooccurence(f2, -2)
    yield c_1

    @raises(ValueError)
    def c_1():
        texture.cooccurence(f3, -2)
    yield c_1

    @raises(ValueError)
    def c_2_10():
        texture.cooccurence(f2, 10)
    yield c_2_10

    @raises(ValueError)
    def c_3_17():
        texture.cooccurence(f3, 17)
    yield c_3_17

    @raises(ValueError)
    def c_4_1():
        texture.cooccurence(f4, 1)
    yield c_4_1



def brute_force(f, dy, dx):
    res = np.zeros((f.max()+1, f.max() + 1), np.double)
    for y in range(f.shape[0]):
        for x in range(f.shape[1]):
            if 0 <= y + dy < f.shape[0] and \
                0 <= x + dx < f.shape[1]:
                res[f[y,x], f[y +dy,x+dx]] += 1
    return res

def brute_force3(f, dy, dx, dz):
    res = np.zeros((f.max()+1, f.max() + 1), np.double)
    for y in range(f.shape[0]):
        for x in range(f.shape[1]):
            for z in range(f.shape[2]):
                if 0 <= y + dy < f.shape[0] and \
                    0 <= x + dx < f.shape[1] and \
                    0 <= z + dz < f.shape[2]:
                    res[f[y,x,z], f[y +dy,x+dx,z+dz]] += 1
    return res


def brute_force_sym(f, dy, dx):
    cmat = brute_force(f, dy, dx)
    return (cmat + cmat.T)

def test_cooccurence():
    np.random.seed(222)
    f = np.random.rand(32, 32)
    f = (f * 255).astype(np.int32)

    assert np.all(texture.cooccurence(f, 0, symmetric=False) == brute_force(f, 0, 1))
    assert np.all(texture.cooccurence(f, 1, symmetric=False) == brute_force(f, 1, 1))
    assert np.all(texture.cooccurence(f, 2, symmetric=False) == brute_force(f, 1, 0))
    assert np.all(texture.cooccurence(f, 3, symmetric=False) == brute_force(f, 1, -1))

    assert np.all(texture.cooccurence(f, 0, symmetric=1) == brute_force_sym(f, 0, 1))
    assert np.all(texture.cooccurence(f, 1, symmetric=1) == brute_force_sym(f, 1, 1))
    assert np.all(texture.cooccurence(f, 2, symmetric=1) == brute_force_sym(f, 1, 0))
    assert np.all(texture.cooccurence(f, 3, symmetric=1) == brute_force_sym(f, 1, -1))

def test_cooccurence3():
    np.random.seed(222)
    f = np.random.rand(32, 32, 8)
    f = (f * 255).astype(np.int32)

    for di, (d0,d1,d2) in enumerate(texture._3d_deltas):
        assert np.all(texture.cooccurence(f, di, symmetric=False) == brute_force3(f, d0, d1, d2))

def test_haralick():
    np.random.seed(123)
    f = np.random.rand(1024, 1024)
    f = (f * 255).astype(np.int32)
    feats = texture.haralick(f)
    assert not np.any(np.isnan(feats))

def test_haralick3():
    np.random.seed(123)
    f = np.random.rand(34, 12, 8)
    f = (f * 255).astype(np.int32)
    feats = texture.haralick(f)
    assert not np.any(np.isnan(feats))


def test_single_point():
    A = np.zeros((5,5), np.uint8)
    A[2,2]=12
    assert not np.any(np.isnan(texture.cooccurence(A,0)))

@raises(TypeError)
def test_float_cooccurence():
    A = np.zeros((5,5), np.float32)
    A[2,2]=12
    texture.cooccurence(A,0)

@raises(TypeError)
def test_float_haralick():
    A = np.zeros((5,5), np.float32)
    A[2,2]=12
    texture.haralick(A)

def test_haralick3d():
    np.random.seed(22)
    img = mahotas.stretch(255*np.random.rand(20,20,4))
    features = texture.haralick(img)
    assert features.shape == (13,13)

    features = texture.haralick(img[:,:,0])
    assert features.shape == (4,13)

    features = texture.haralick(img.max(0), ignore_zeros=True, preserve_haralick_bug=True, compute_14th_feature=True)
    assert features.shape == (4,14)


def test_zeros():
    zeros = np.zeros((64,64), np.uint8)
    feats = texture.haralick(zeros)
    assert not np.any(np.isnan(feats))

    feats = texture.haralick(zeros, ignore_zeros=True)
    assert not np.any(np.isnan(feats))

@raises(ValueError)
def test_4d_image():
    texture.haralick(np.arange(4**5).reshape((4,4,4,4,4)))


def rand_haralick():
    f = 255*np.random.random((128,128))
    f = f.astype(np.uint8)
    f = mh.features.haralick(f)
    return f.mean(0)
def test_feature_non_zero():
    np.random.seed(23)
    assert any(np.all(rand_haralick() != 0) for i in range(12))

def test_feature_not_same():
    np.random.seed(26)

    multiple = np.array([rand_haralick() for i in range(8)])
    assert np.all(multiple.ptp(0) > 0)



def test_return_mean_ptp():
    f = 255*np.random.random((128,128))
    f = f.astype(np.uint8)
    fs = mh.features.haralick(f)
    fs_mean = mh.features.haralick(f, return_mean=1)
    fs_mean_ptp = mh.features.haralick(f, return_mean_ptp=1)
    assert np.all(fs.mean(0) == fs_mean)
    assert np.all(fs.mean(0) == fs_mean_ptp[:fs.shape[1]])
    assert np.all(fs.ptp(0) == fs_mean_ptp[fs.shape[1]:])

@raises(ValueError)
def test_return_mean_ptp_xor():
    f = 255*np.random.random((128,128))
    f = f.astype(np.uint8)
    fs = mh.features.haralick(f, return_mean=1, return_mean_ptp=1)


def test_x_minus_y():
    f = 255*np.random.random((128,128))
    f = f.astype(np.uint8)
    h = mh.features.haralick(f, use_x_minus_y_variance=False)
    h2 = mh.features.haralick(f, use_x_minus_y_variance=True)
    assert np.sum(h != h2) == 4
